"""
Typing for the operator string a user can provide. 

This file is autogenerated based on the `operators` GraphQL query. 
"""

from typing import Literal

OperatorType = Literal[
    "includes",
    "paragraph_format",
    "matches_json_schema",
    "satisfies_statement",
    "equals_exactly",
    "legacy_equals",
    "is_not_hallucinating",
    "list_format",
    "answers",
    "not_answers",
    "valid_json",
    "constitutional",
    "no_legal_advice",
    "excludes",
    "consistent_with_context",
    "consistent_with_json",
    "affirmative_answer",
    "golden_check_style",
    "matches_tone",
    "golden_check_content",
    "is_concise",
    "is_not_generated",
    "is_safe",
    "less_than_length",
    "equals",
    "is_coherent",
    "consistent_with_docs",
    "includes_nearly_exactly",
    "negative_answer",
    "logical_includes",
    "includes_exactly",
    "equal_intent",
    "regex",
    "consistent_phrasing",
    "includes_any",
    "safety_consistency",
    "excludes_exactly",
    "progresses_conversation",
    "includes_only",
    "includes_any_v2",
    "valid_yaml",
    "grammar",
    "custom_operator",
    "is_polite",
    "capitalization_check",
    "golden_check_format",
    "matches_format",
]
