# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Optional

from pydantic import Field

from .base_model import BaseModel


class UpdateGlobalChecks(BaseModel):
    update_global_checks: Optional["UpdateGlobalChecksUpdateGlobalChecks"] = Field(
        alias="updateGlobalChecks"
    )


class UpdateGlobalChecksUpdateGlobalChecks(BaseModel):
    success: bool


UpdateGlobalChecks.model_rebuild()
