# Generated by ariadne-codegen
# Source: http://localhost:8000/graphql/

from enum import Enum


class AppQuestionAnswerSetCreationMethodChoices(str, Enum):
    LIVE_QUERY = "LIVE_QUERY"
    SDK = "SDK"
    UPLOAD = "UPLOAD"


class RunStatus(str, Enum):
    IN_PROGRESS = "IN_PROGRESS"
    ERROR = "ERROR"
    SUCCESS = "SUCCESS"


class TestSuiteSortField(str, Enum):
    TITLE = "TITLE"
    CREATED = "CREATED"
    LAST_MODIFIED_AT = "LAST_MODIFIED_AT"


class SortOrder(str, Enum):
    ASC = "ASC"
    DESC = "DESC"


class RunResultSortField(str, Enum):
    NAME = "NAME"
    STARTED_AT = "STARTED_AT"
    COMPLETED_AT = "COMPLETED_AT"
    PASS_PERCENTAGE = "PASS_PERCENTAGE"


class TierEnum(str, Enum):
    FREE = "FREE"
    STARTUP = "STARTUP"
    ENTERPRISE = "ENTERPRISE"
