# Generated by ariadne-codegen
# Source: vals/graphql/

from .base_model import BaseModel


class RunStatus(BaseModel):
    run: "RunStatusRun"


class RunStatusRun(BaseModel):
    status: str


RunStatus.model_rebuild()
