# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Optional

from pydantic import Field

from .base_model import BaseModel


class DeleteTestSuite(BaseModel):
    delete_suite: Optional["DeleteTestSuiteDeleteSuite"] = Field(alias="deleteSuite")


class DeleteTestSuiteDeleteSuite(BaseModel):
    success: bool


DeleteTestSuite.model_rebuild()
