# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Any, List

from pydantic import Field

from .base_model import BaseModel


class GetTestData(BaseModel):
    tests: List["GetTestDataTests"]


class GetTestDataTests(BaseModel):
    checks: Any
    test_id: str = Field(alias="testId")
    cross_version_id: str = Field(alias="crossVersionId")
    file_ids: Any = Field(alias="fileIds")
    input_under_test: str = Field(alias="inputUnderTest")
    tags: Any
    context: Any
    golden_output: str = Field(alias="goldenOutput")
    test_suite: "GetTestDataTestsTestSuite" = Field(alias="testSuite")


class GetTestDataTestsTestSuite(BaseModel):
    id: str


GetTestData.model_rebuild()
GetTestDataTests.model_rebuild()
