mutation CreateQuestionAnswerSet(
  $testSuiteId: String!
  $questionAnswerPairs: [QuestionAnswerPairInputType!]!
  $parameters: GenericScalar!
  $modelId: String!
) {
  createQuestionAnswerSet(
    testSuiteId: $testSuiteId
    questionAnswerPairs: $questionAnswerPairs
    parameters: $parameters
    modelId: $modelId
  ) {
    questionAnswerSet {
      id
    }
  }
}

mutation BatchAddQuestionAnswerPairs(
  $questionAnswerSetId: String!
  $questionAnswerPairs: [QuestionAnswerPairInputType!]!
) {
  batchAddQuestionAnswerPairs(
    questionAnswerSetId: $questionAnswerSetId
    questionAnswerPairs: $questionAnswerPairs
  ) {
    questionAnswerPairs {
      id
    }
  }
}
